/**********************************************************
***********************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:  memoryHandling.h
* Description:
*
*
* History:
* 23-Jun-03     (csabet) Created
***********************************************************
**********************************************************/
#ifndef _MEMORY_HANDLING_H
#define _MEMORY_HANDLING_H

/**********************************************************
*   Include files
**********************************************************/

/**********************************************************
*   Constant definitions
**********************************************************/

/**********************************************************
*   Global Variables
**********************************************************/

/**********************************************************
*   Type definitions
**********************************************************/

/**********************************************************
*   Function prototypes
**********************************************************/
int **callocRejectList (
    ImageFormat	*format);

void freeRejectList (
    ImageFormat	*format,
    int			**array);

KappaCoefficients *callocKappaCoefficients (
	int	numOfChannels);

void freeKappaCoefficients (
	KappaCoefficients	*array);

TransferRatios *callocTransferRatios (
	int	numOfChannels);

void freeTransferRatios (
	TransferRatios	*array);

ImageQuality *callocImageQuality (
    ImageFormat	*format);

void freeImageQuality (
    ImageFormat		*format,
    ImageQuality	*image);

ImageFormat *callocImageFormat (void);

void freeImageFormat (
    ImageFormat *array);

FilterData *callocFilterInfo (void);

void freeFilterInfo (
    FilterData *array);

CalibratedVisibility *callocCalibVis (void);

void freeCalibVis (
    CalibratedVisibility *array);

TransferFunction *callocTrFunction (
	int	size);

void freeTrFunction (
    TransferFunction *array);

void freeCalibrator (
    CalibratorParam *calibrator);

CalibratorParam *callocCalibrator (void);

IauExchange *callocIauExchange (
    int numOfTelescopes,
    int	numOfChannels);

void freeIauExchange (
    IauExchange *iauData);

PhotometryResult *callocPhotometry (void);

void freePhotometry (
PhotometryResult *photometry);

RawVisibility *callocRawVis (void);

void freeRawVis (
    RawVisibility *rawVis);

CompressedData *callocMidiCompressed (
    ImageFormat			*format);

void freeCompressedData (
    ImageFormat			*format,
    CompressedData		*arrayData);

MidiFiles *callocMidiFiles (
    int stringLength);

void freeMidiFiles (
    MidiFiles   *fileNmaes);

float **callocWaveCalibration (
	int			regions,
    ImageFormat	*format);

void freeWaveCalibration (
	int		numOfRegions,
	float	**array);

DispersedResult *callocDispResult (
	int	numOfChannels);

void freeDispResult (
	DispersedResult	*array);

DispersiveTrans *callocDspTrn (
    int	numOfFiles);

void freeDspTrn (
    DispersiveTrans	*array);

DetLinearity *callocDetLin (
    int	numOfFiles,
    ImageFormat	*format);

void freeDetLin (
    DetLinearity	*array);

DetRonNoise *callocDetRon (
    ImageFormat	*imageSize);

void freeDetRon (
    DetRonNoise	*array);

ReferencePixel *callocRefPix (
    int			numOfFiles,
    int			numOfBeams);
    
void freeRefPix (
    ReferencePixel	*array);

WaveCalibration *callocWaveCal (
    int			numOfFiles,
    int			numOfRecords,
    ImageFormat	*format);

void freeWaveCal (
    ImageFormat		*format,
    WaveCalibration	*array);

/*********************************************************/
/*********************************************************/
#endif
