/**********************************************************
***********************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  midiLib.h
* Description:
*
*
* History:
* 03-Mar-03     (csabet) Created
***********************************************************
**********************************************************/
#ifndef _MIDI_LIB_H
#define _MIDI_LIB_H

/**********************************************************
*   Include files
**********************************************************/
#include "iauExchange.h"

/**********************************************************
*   Constant definitions
**********************************************************/

/**********************************************************
*   Enumeration definitions
**********************************************************/
/*  Mask Mode */
enum MaskMode
{
    COMPUTED,
    SELECTED,
    SPECIFIED
};

/*  Obs Technique */
enum ObsTechnique
{
    INTERF,
    PHOTOMA,
    PHOTOMB,
    UNKNOWN
};

/*  Processing Modes */
enum ProcessingMode
{
    UNDISPERSED,
    DISPERSED,
    DISPERSED_SLIT,
    DISPERSED_3PH
};

/**********************************************************
*   Type definitions
**********************************************************/
typedef struct
{
    enum MaskMode       maskMode;
    enum ProcessingMode	processing;
    int	 drsFrg;  // 0, 1, 2: Decativate, Activate
    int	 drsKap;  // 0, 1, 2: Decativate, Read, Update
    int	 drsAcq;  // 0, 1, 2: Decativate, Activate
    int	 tecRon;  // 0, 1, 2: Decativate, Activate
    int	 tecLin;  // 0, 1, 2: Decativate, Activate
    int	 tecPix;  // 0, 1, 2: Decativate, Activate
    int	 tecTrn;  // 0, 1, 2: Decativate, Activate
    int	 tecWav;  // 0, 1, 2: Decativate, Create, Correlate
} UserOptions;

typedef struct
{
    OiArray      *array;
    OiTarget     *targets;
    OiWavelength *wavelength;
    OiVis        *vis;
    OiVis2       *vis2;
} IauExchange;

typedef struct
{
    char *inFitsList;
    char *inFitsClassified;
    char *inFitsBatch;
    char *inFileDir;
    char *inFitsName;
    char *maskFileDir;
    char *maskFileName;
    char *outFitsName;
    char *outWaterfallName;
    char *outWaterpowerName;
    char *outAveImageName;
    char *outChSpectrumName;
    char *calibDbName;
    char *waveCalibName;
    char *grismFoilName;
    char *prismFoilName;
    char *calibDbDir;
    char *trfNameWrite;
    char *trfNameRead;
    char *trfHistoryName;
    char *trrNameWrite;
    char *trrNameRead;
    char *trrHistoryName;
    char *wclNameWrite;
    char *wclNameRead;
    char *transferRatiosName;
    char *wclHistoryName;
    char *archFileName;
    char *pipeFileName;
} MidiFiles;

//  Fits file format
typedef struct
{
    int	  chopped;
    int	  hasData;
    int	  isDisp;
    int	  numOfFrames;		// Total number of frames in a batch. used in UNDISPERSED
    int	  framesPerScan;
    int	  numOfScans;  		// numOfScans = numOfFrames / framesPerScan;
    int	  numOfPinholes;  	// 1 standard; 3 if triple pinhole; 2 if slit with sky definition
    int	  numOfDetectorRegions;	// In raw data from detector. Number of Detector regions
    int	  numOfRegionsToProcess;// Number that get processed
    int	  iXWidth;		// X dimension
    int	  iYWidth;		// Y dimension
    int	  subWindowSize;   	// iXWidth * iYWidth;
    int	  numOfTemplates;
    int	  numOfFilesInTemplate;
    int	  fileNumInTemplate;
    int	  numOfExposures;
    int	  fileNumInExposure;
    int	  exposureNumInTemplate;
    int	  fftsize;
    int	  allSpectrumLength;
    float optFreqCal;
    int	  numOfScansRejected;
    int	  numOfScansProcessed;
    int	  numOfChannelsProcessed;
    int	  numOfChannelsRejected;
    int	  numOfSkyScans;
    char  *beamCombiner;
    char  *tplName;
    char  *obsTech;
    char  *obsCatg;
    char  *obsType;
    char  *shutterId;
    char  *grismId;
    char  *targetName;
    char  *tplStart;
    char  *cameraId;
    char  *slitName;
    char  *filterName;
} ImageFormat;

//  Transfer Ratios
typedef struct
{
    int	  exists;
    float *ka1;
    float *siga1;
    float *ka2;
    float *siga2;
    float *kb1;
    float *sigb1;
    float *kb2;
    float *sigb2;
} TransferRatios;

//  Kappa Coefficients
typedef struct
{
    float *k11;
    float *sig11;
    float *k21;
    float *sig21;
    float *k31;
    float *sig31;
    float *k22;
    float *sig22;
    float *k32;
    float *sig32;
    float *k42;
    float *sig42;
} KappaCoefficients;

//  Only for DISPERSED
typedef struct
{
    int	  photomAExists;
    int	  photomBExists;
    int	  photomIExists;
    int	  trfExists;
    int	  calibVisExists;
    float *rawVis2;
    float *rawVis2Err;
    float *normVis2;
    float *normVis2Err;
    float *calibVis2;
    float *calibVis2Err;
    float **photomA;
    float **photomB;
    float **photomI;
    float **photomAErr;
    float **photomBErr;
    float **photomIErr;
    float *photomNet2;	  // Net photometry estimate
    float *photomNet2Err; // Net photometry error 
    float *trf;
    float *trfErr;
    float photomATarg1;
    float photomATarg2;
    float photomATarg3;
    float photomBTarg1;
    float photomBTarg2;
    float photomBTarg3;
    float photomATotal1;
    float photomATotal2;
    float photomATotal3;
    float photomBTotal1;
    float photomBTotal2;
    float photomBTotal3;
    float uncalVisBinned1;
    float uncalVisBinned2;
    float uncalVisBinned3;
    float uncalVisBinned4;
    float uncalVisBinned5;
    float trfBinned1;
    float trfBinned2;
    float trfBinned3;
    float trfBinned4;
    float trfBinned5;
} DispersedResult;

typedef struct
{
    int	   exists;
    float  ***iDispFringe;    // Triple pointer: regionsToProcess, Wave (X), time (frame)
    double *iFringe;
    double *iFringe1;
    double *iFringe2;
    float  *allSpectrum;
    float  **image;	      // Region, image
    float  *dcLevels;
    char   *tarType;
    int	   numOfTargetChops;
    float  *time;		// Array of framelength, time of each in JD relative to timezero
    float  *localOPD;		// Array of framelength, net Piezo OPD
    float  *bigDL; 		// Array of framelength, net Piezo OPD
    int	   **rejectList;	// 2D array of numOfChannels and numOfFrames
    int	   *badScanList;	// Array of numOfScans
} CompressedData;

typedef struct
{
    float   optFreqLo;
    float   optFreqHi;
    char    *filterName;
} FilterData;

//  Only for UNDISPERSED
typedef struct
{
    float   vis;
    float   visErr;
    float   vis1;
    float   vis1Err;
    float   vis2;
    float   vis2Err;
    float   visSqrd;
    float   visSqrdErr;
    float   visSqrd1;
    float   visSqrd1Err;
    float   visSqrd2;
    float   visSqrd2Err;
} RawVisibility;

//  Only for UNDISPERSED
typedef struct
{
    int	  exists;
    float *vis;
    float *visErr;
    float *visSqrd;
    float *visSqrdErr;
} CalibratedVisibility;

//  Only for UNDISPERSED
typedef struct
{
    int	  exists;
    float *trf;
    float *trfErr;
} TransferFunction;

//  Only for UNDISPERSED
typedef struct
{
    float   *photomA;
    float   *photomAErr;
    float   *photomB;
    float   *photomBErr;
    float   ratioPhotomA0toA1;
    float   ratioPhotomB0toB1;
    float   *chop2ChopAErr;
    float   *chop2ChopBErr;
} PhotometryResult;

typedef struct
{
    double  uCoord;
    double  vCoord;
    double  wCoord;
} UVW;

typedef struct
{
    char  *calibName;
    float calibRA;
    float calibDEC;
    float calibDistance;
    float calibVis;
    float calibVisErr;
    float calibVisSqrd;
    float calibVisSqrdErr;
    float calibPblAverage;
    float calibParangAverage;
    float calibDiameter;
    float calibDiameterErr;
    double calibFlux;
    int   calibFlag;
} CalibratorParam;

typedef struct
{
    double xCoord;	
    double yCoord;	
    double dxCoord;	
    double dyCoord;	
    double size;
} MidiCoords;

typedef struct
{
    int	   exists;
    float  **aveImage;
    float  **aveImageSky;
    double *coordX;
    double *coordY;
    double *sizeX;
    double *sizeY;
    float  *targetFlux;
    int	   *targetPixelCount;
    char   *tarType;
} ImageQuality;


typedef struct
{
    int	   exists;
    float  *pattern;
    float  *aveImage;
    double median;
} DetRonNoise;

typedef struct
{
    int	  exists;
    int	  size;
    float *mean;
    char  *grismId;
    int	  *saturated;
    float **aveImage;
    float *integTime;
    float *deviation;
    float meanCoeffA0;
    float meanCoeffA1;
    float meanCoeffA2;
    float meanCoeffA3;
    float meanSigma;
    int	  winx;
    int	  windx;
    int	  winy;
    int	  windy;
} DetLinearity;

typedef struct
{
    float *xCoord;
    float *yCoord;
    float *wavelength;
    char  *fileName;
    float **image;
} NarrowBandInfo;

typedef struct
{
    float *actualWavelength;
    float *actualTransmission;
    char  *fileName;
    float **image;
} FoilInfo;

typedef struct
{
    char  *fileName;
    float **image;
} OpenInfo;

typedef struct
{
    char  *fileName;
    float **image;
} DarkInfo;

typedef struct
{
    int	           exists;
    int	           numOfFiles;
    char           **filterName;
    char           **shutterId;
    char	   *tarType;
    float	   **calibratedWave;
    float	   *variance;
    NarrowBandInfo *ArIII;
    NarrowBandInfo *NeII;
    NarrowBandInfo *SIV;
    FoilInfo	   *foil;
    OpenInfo       *open;
    DarkInfo       *dark;
} WaveCalibration;

typedef struct
{
    int		numOfBeams;
    char	*shutterId;
    char	*beamCombiner;
    MidiCoords	*centroid;
} RefPixExposure;

typedef struct
{
    int		   exists;
    int		   numOfExposures;
    RefPixExposure *exposure;
} ReferencePixel;

typedef struct
{
    int    	exists;
    char   	**beamCombiner;
    char   	**shutterId;
    char	**cameraId;
    char	**grismId;
    char	**filterName;
    int		numOfFiles;
    MidiCoords	*target;
    float	*integTime;
    float	*integFlux;
    float	*meanFlux;
    float	*transmission;
} DispersiveTrans;

typedef struct
{
    long	CorrInterf;
    long	CorrPhotomA;
    long	CorrPhotomB;
} CorrectedFrames;

/**********************************************************
*   Global Variables
**********************************************************/

/**********************************************************
*   Function Prototypes
**********************************************************/
float sqrtp (float x);
float sqr(float x);
void Trace (int n);

/*********************************************************/
#endif
/*********************************************************/
/*********************************************************/
